import {Component, Input, OnChanges} from '@angular/core';
import {MatTableDataSource, MatTableModule} from "@angular/material/table";
import {MatIcon} from "@angular/material/icon";
import {DatePipe} from "@angular/common";
import {addDays} from "@progress/kendo-date-math";
import {MatDivider} from "@angular/material/divider";
import {DataSpinnerComponent} from "@component/common/data-spinner/data-spinner.component";
import {ScheduleResponse} from "@api/v1/scheduleResponse";

interface TableData {
  pairNumber: number;
  data: Dictionary;
}

interface Dictionary {
  [key: string]: any;
}

@Component({
  selector: 'app-table',
  standalone: true,
  imports: [
    MatTableModule,
    MatIcon,
    DatePipe,
    MatDivider,
    DataSpinnerComponent
  ],
  templateUrl: './table.component.html',
  styleUrl: './table.component.css',
})

export class TableComponent implements OnChanges {
  private isDisciplineWithWeeks: boolean = false;
  protected tableDataSource: MatTableDataSource<TableData> = new MatTableDataSource<TableData>([]);
  private backupDisciplines: string[] = [];
  protected daysOfWeek: string[] = ['Понедельник', 'Вторник', 'Среда', 'Четверг', 'Пятница', 'Суббота'];
  protected displayedColumns: string[] = ['pairNumber'];
  protected dataSource: ScheduleResponse[] = [];
  protected isOneGroup: boolean = false;

  @Input() currentWeek!: number;
  @Input() startWeek!: Date;
  @Input() isLoad: boolean = false;

  @Input() set disciplineWithWeeks(value: boolean) {
    this.isDisciplineWithWeeks = value;
    this.convertData();
  }

  @Input() set data(schedule: ScheduleResponse[]) {
    this.dataSource = schedule;
    this.convertData();
    this.isOneGroup = schedule.every((item, _, array) => item.group === array[0].group);
  }

  ngOnChanges(changes: any) {
    if (changes.startWeek && !changes.startWeek.firstChange) {
      this.convertData();
    }
  }

  constructor() {
    this.displayedColumns = this.displayedColumns.concat(this.daysOfWeek);
  }

  private convertData() {
    this.isLoad = true;
    let tableData: TableData[] = [];

    for (let i: number = 1; i <= 7; i++) {
      let convertedData: TableData = {
        pairNumber: i,
        data: {}
      };

      for (let k: number = 1; k < 7; k++) {
        let filteredData = this.dataSource.filter(x =>
          x.pairNumber === i &&
          x.dayOfWeek === k &&
          x.isEven === (this.currentWeek % 2 === 0)
        );

        if (!this.isDisciplineWithWeeks)
          filteredData = filteredData.filter(x =>
            x.isExcludedWeeks == undefined ||
            x.weeks == undefined ||
            (x.isExcludedWeeks && (!x.weeks.includes(this.currentWeek))) ||
            (!x.isExcludedWeeks && (x.weeks.includes(this.currentWeek)))
          );

        filteredData.forEach(x => {
          if (this.isDisciplineWithWeeks) {
            if (x.isExcludedWeeks != undefined && x.weeks != undefined) {
              if (this.backupDisciplines[x.disciplineId])
                x.discipline = this.backupDisciplines[x.disciplineId];
              else
                this.backupDisciplines[x.disciplineId] = x.discipline;

              x.discipline = `${(x.isExcludedWeeks ? 'кр.' : 'н.')} ${x.weeks.join(', ')} ${x.discipline}`;
            }
          } else if (this.backupDisciplines[x.disciplineId])
            x.discipline = this.backupDisciplines[x.disciplineId];
        });

        convertedData.data[k.toString()] = filteredData;
      }

      tableData.push(convertedData);
    }

    this.tableDataSource = new MatTableDataSource<TableData>(tableData);
    this.isLoad = false;
  }

  protected checkAvailableData(data: any[] | any): boolean {
    return data && data.length !== 0 && (!Array.isArray(data) || data.every(x => x !== null));
  }

  protected readonly addDays = addDays;
}
